import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

public class PromiseTest {
	public static class Promise<T> {
		private T value;
		private Object lock = new Object();
		private boolean hasValue = false;
		
		public T getValue() {
			while (!hasValue) {
				synchronized (lock) {
					try {
						lock.wait();
					} catch (InterruptedException e) {
						e.printStackTrace();
					};
				}
			}
			return value;
		}
		
		public void setValue(T val) {
			value = val;
			hasValue = true;
			synchronized (lock) {
				lock.notifyAll();
			}
		}
	}
	
	private static void start(Runnable r) {
		new Thread(r).start();
	}
	
	public static void main(String[] args) {
	final Promise<Double> m = new Promise<Double>();
	final Promise<Double> e = new Promise<Double>();
	final Promise<Double> h = new Promise<Double>();
	final double C = 299792458; 
	
	start(new Runnable() {
		public void run() {
			e.setValue(m.getValue()*C*C);
			System.out.println("Your energy is "+e.getValue()+" J");
		}
	});
	start(new Runnable() {
		public void run() {
			h.setValue(Math.sqrt(m.getValue()/25.0));
			System.out.println("You should be at least "+h.getValue()+" m tall.");
		}
	});
	start(new Runnable() {
		public void run() {
			BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
			System.out.print("Your mass in kg:");
			try {
				m.setValue(Double.valueOf(br.readLine()));
			} catch (IOException e) {					
				e.printStackTrace();
			}
		}
	});
	}
}
